/* Nagios Dock Icon for Mac OS X 
	Michael Baysek, May 2005
    This is an adaptation of bsTray for use with Nagios.  I have added a few other things to it, such
	as being able to contol get bouncing dock icon when status changes.
*/


import com.apple.cocoa.foundation.*;
import com.apple.cocoa.application.*;
import java.net.*;
import java.util.*;
import java.io.*;

public class BSTray {

	protected static NSImage imgRed = null;
	protected static NSImage imgGreen = null;
	protected static NSImage imgYellow = null;
	protected static NSImage imgPurple = null;

	protected static NSSound sndRed = null;
	protected static NSSound sndGreen = null;
	protected static NSSound sndYellow = null;
	protected static NSSound sndPurple = null;

	protected static Properties iniProperties = null;
	protected static int pollFrequency = 15;
	protected static String displayURL = null;
	protected static String displayPassword = null;
	protected static boolean bounceOnNotify = false;
	protected static int lastStatus=0;


	static 
	{

		String imgRedPath = NSBundle.mainBundle().pathForResource("red", "jpg");
		if (imgRedPath != null)
		{
			imgRed = new NSImage(imgRedPath, true);
		}
		else
		{
			NSSystem.log("Image red.jpg not found");
		}
		String imgYellowPath = NSBundle.mainBundle().pathForResource("yellow", "jpg");
		if (imgYellowPath != null)
		{
			imgYellow = new NSImage(imgYellowPath, true);
		}
		else
		{
			NSSystem.log("Image yellow.jpg not found");
		}
		String imgGreenPath = NSBundle.mainBundle().pathForResource("green", "jpg");
		if (imgGreenPath != null)
		{
			imgGreen = new NSImage(imgGreenPath, true);
		}
		else
		{
			NSSystem.log("Image green.jpg not found");
		}
		String imgPurplePath = NSBundle.mainBundle().pathForResource("purple", "jpg");
		if (imgPurplePath != null)
		{
			imgPurple = new NSImage(imgPurplePath, true);
		}
		else
		{
			NSSystem.log("Image purple.jpg not found");
		}

		String sndRedPath = NSBundle.mainBundle().pathForResource("r", "wav");
		if (sndRedPath != null)
		{
			sndRed = new NSSound(sndRedPath, true);
		}
		else
		{
			NSSystem.log("Sound r.wav not found");
		}
		String sndYellowPath = NSBundle.mainBundle().pathForResource("y", "wav");
		if (sndYellowPath != null)
		{
			sndYellow = new NSSound(sndYellowPath, true);
		}
		else
		{
			NSSystem.log("Sound y.wav not found");
		}
		String sndGreenPath = NSBundle.mainBundle().pathForResource("g", "wav");
		if (sndGreenPath != null)
		{
			sndGreen = new NSSound(sndGreenPath, true);
		}
		else
		{
			NSSystem.log("Sound g.wav not found");
		}
		String sndPurplePath = NSBundle.mainBundle().pathForResource("p", "wav");
		if (sndPurplePath != null)
		{
			sndPurple = new NSSound(sndPurplePath, true);
		}
		else
		{
			NSSystem.log("Sound p.wav not found");
		}

		String propertiesPath = NSBundle.mainBundle().pathForResource("NagiosDock", "properties");
		if (propertiesPath != null)
		{
			try
			{
				File propFile = new File(propertiesPath);
				FileInputStream fis = new FileInputStream(propFile);
				iniProperties = new Properties();
				iniProperties.load(fis);
				initProperties();
			}
			catch(Exception ex)
			{
				NSSystem.log("NagiosDock.properties was not found");
			}
		}
		else
		{
			NSSystem.log("NagiosDock.properties was not found");
		}



	}
	


	NSImage imgCurrent = null;

    public void updateNow(Object sender) { /* IBAction */
		try
		{
			checkDisplayServer();
		}	
		catch(Exception ex)
		{
			setApplicationIconImage(imgPurple);
			NSSystem.log(ex.toString());
		}
	
	 
    }
	
    public void openDisplayURL(Object sender) { /* IBAction */
		try
		{
			NSWorkspace.sharedWorkspace().openURL(new URL(displayURL));
		}
		catch(Exception ex)
		{
			NSSystem.log(ex.toString());
		}
	
	 
    }

    public void editBSTrayIni(Object sender) { /* IBAction */
		try
		{
			String propertiesPath = NSBundle.mainBundle().pathForResource("NagiosDock", "properties");
			NSWorkspace.sharedWorkspace().openFile(propertiesPath,"textedit",true);
		}
		catch(Exception ex)
		{
			NSSystem.log(ex.toString());
		}
	
	 
    }


	public void awakeFromNib() 
	{
			try
			{
				initIcon();
				loop();
			}
			catch(Exception ex)
			{
				NSSystem.log(ex.toString());
			}
			
	}

	protected static void playSound(String wavfilename)
	{
		if(wavfilename==null)
		{
			return;
		}
		
		if(wavfilename.equals("g.wav")&&sndGreen!=null)
		{
			sndGreen.play();
		}
		else if(wavfilename.equals("y.wav")&&sndYellow!=null)
		{
			sndYellow.play();
		}
		else if(wavfilename.equals("r.wav")&&sndRed!=null)
		{
			sndRed.play();
		}
		else if(wavfilename.equals("p.wav")&&sndPurple!=null)
		{
			sndPurple.play();
		}
		

		
	}

	protected void setApplicationIconImage(NSImage img)
	{
			NSApplication app = NSApplication.sharedApplication(); 
			if(imgCurrent!=img)
			{
				imgCurrent = img;
				app.setApplicationIconImage(imgCurrent);
				if(imgCurrent == imgGreen)
				{
					if (lastStatus > 0) {
						if (bounceOnNotify) { app.requestUserAttention(NSApplication.UserAttentionRequestInformational); }
						playSound("g.wav");
					}
					lastStatus = 0;
					
				}
			}

			if(imgCurrent == imgYellow)
			{
				if (lastStatus != 1) {
					if (bounceOnNotify) { app.requestUserAttention(NSApplication.UserAttentionRequestCritical); }
					playSound("y.wav");
				}
				lastStatus = 1;
			}
			else if(imgCurrent == imgRed)
			{
				if (lastStatus != 2 ) {
					if (bounceOnNotify) { app.requestUserAttention(NSApplication.UserAttentionRequestCritical); }
					playSound("r.wav");
				}
				lastStatus = 2;
			}
			else if(imgCurrent == imgPurple)
			{
				if (bounceOnNotify) { app.requestUserAttention(NSApplication.UserAttentionRequestCritical); }
				playSound("p.wav");
			}
	}
	
	protected void loop() throws Exception
	{
	
			Runnable r = new Runnable() {
				public void run()
				{
					while(true)
					{
							
						try
						{
							checkDisplayServer();
						}	
						catch(Exception ex)
						{
							setApplicationIconImage(imgPurple);
							NSSystem.log(ex.toString());
						}

						int mydelay = pollFrequency * 1000;
						try	{	Thread.sleep(mydelay); } catch (InterruptedException ex) {}
					}
				}
			};
			Thread worker = new Thread(r);
			worker.setPriority(Thread.MIN_PRIORITY);
			worker.start();
			
	}

	protected static void initProperties()
	{
		String strPollFrequency = (String)iniProperties.get("PollFrequency");
		if(strPollFrequency==null)
		{
			strPollFrequency = "15";
		}
		try
		{
			pollFrequency = new Integer(strPollFrequency).intValue();
		}
		catch(NumberFormatException ex)	{}
		if(pollFrequency<15)
		{
			pollFrequency = 15;
		}

		displayURL = (String)iniProperties.get("DisplayURL");
		displayPassword = (String)iniProperties.get("Password");
		String strBounce = (String)iniProperties.get("BounceOnNotify");
		if (strBounce == "true") {
			bounceOnNotify = true;
		} else {
			bounceOnNotify = false;
		}
		
		
	}


	protected void checkDisplayServer() throws Exception
	{

				URL url = new URL(displayURL);

				HttpURLConnection conn = (HttpURLConnection)url.openConnection();
				if(displayPassword!=null&&displayPassword.length()>0)
				{
					conn.setRequestProperty( "Authorization", "Basic " + Base64.encodeString(displayPassword));
				}
				InputStream in = conn.getInputStream();

				String result = "";
				byte buffer[] = new byte[1000];
				while ((in.read(buffer)) != -1) 
				{
					result = result + "" + new String(buffer);
					buffer = new byte[1000];
				}				
				
				conn.disconnect();
				//NSSystem.log(result);

				int greenCount = findCount(result,"OK");
				int redCount = findCount(result,"DOWN") + findCount(result,"CRITICAL");
				int yellowCount = findCount(result,"WARNING") + findCount(result, "UNREACHABLE");
				int purpleCount = findCount(result,"purple.gif");
				
				NSImage img = imgPurple;
				
				if(greenCount>0)
				{
					img = imgGreen;
				}

				if(yellowCount>0)
				{
					img = imgYellow;
				}
				
				if(purpleCount>0)
				{
					img = imgPurple;
				}
	
				if(redCount>0)
				{
					img = imgRed;
				}
				
				
				setApplicationIconImage(img);
		
	}

	protected static int findCount(String inString, String find)
	{
		int ret = 0;
		if (inString==null || find == null||inString.length() == 0)
		{
			return ret;
		}

		int i = inString.indexOf(find);
		
		while(i<inString.length()&&i!=-1)
		{
			ret++;
			i = inString.indexOf(find,i+1);
		   
		}
		return ret;

	}

	protected void initIcon() throws Exception
	{
			NSApplication app = NSApplication.sharedApplication(); 
			app.setApplicationIconImage(imgPurple);

	}











}
